﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using Microsoft.Win32;

namespace XE
{
    public partial class Form1 : Form
    {

        List<CurrencyMatch> _matches = new List<CurrencyMatch>();

        public Form1()
        {
            InitializeComponent();
            GetValuesFromRegistry();
        }
        
        private void timer1_Tick(object sender, EventArgs e)
        {
            DoTheFrigginWork();
        }

        private void DoTheFrigginWork()
        {
            txtOutput.Text += String.Format("Pulling rates at {0}\r\n", DateTime.Now);
            string htmlResponse = GetHtml();

            foreach (CurrencyMatch match in _matches)
            {

                GetRates(match, htmlResponse);
                if (match.CurrentPercentageDifference > match.MinEntryPercentageDifference && !match.EntryAlertHasBeenIssued)
                {
                    IssueEntryAlert(match);
                }
                else if (match.CurrentPercentageDifference < match.MinEntryPercentageDifference && match.EntryAlertHasBeenIssued)
                {
                    IssueExitAlert(match);
                }
                match.CurrentValueLabel.Text = String.Format("[{0}/{1}]=%{2}", 
                    match.Currency1.CurrentValue, 
                    match.Currency2.CurrentValue, 
                    Decimal.Round(match.CurrentPercentageDifference, 4).ToString());
            }
        }

        private void GetRates(CurrencyMatch match, string htmlResponse)
        {
            string currency1String = htmlResponse.Substring(htmlResponse.IndexOf(match.Currency1.StartString), match.Currency1.SubStringLength);
            string currency2String = htmlResponse.Substring(htmlResponse.IndexOf(match.Currency2.StartString), match.Currency2.SubStringLength);

            match.Currency1.CurrentValue = FindLastDecimalValue(currency1String);
            match.Currency2.CurrentValue = FindLastDecimalValue(currency2String);

        }

        private string GetHtml()
        {
            WebRequest request = HttpWebRequest.Create("http://www.xe.com");
            WebResponse response = request.GetResponse();
            System.IO.Stream responseStream = response.GetResponseStream();
            System.IO.StreamReader sr = new System.IO.StreamReader(responseStream);
            string htmlResponse = sr.ReadToEnd();
            return htmlResponse;
        }

        private Decimal FindLastDecimalValue(string input)
        {
            List<Decimal> decimalValues = new List<decimal>();
            string currentDecimalValue = String.Empty;
            foreach (Char c in input.ToCharArray())
            {
                if (Char.IsNumber(c) || c == '.')
                {
                    currentDecimalValue += c;
                }
                else
                {
                    if (!String.IsNullOrEmpty(currentDecimalValue))
                        decimalValues.Add(Convert.ToDecimal(currentDecimalValue));
                    currentDecimalValue = String.Empty;
                }
            }
            if (decimalValues.Count > 0)
                return decimalValues[decimalValues.Count - 1];
            else
                throw new Exception(String.Format("No decimal values found in {0}", input));
        }

        private void IssueEntryAlert(CurrencyMatch match)
        {
            txtOutput.Text += String.Format("Entry: {0}[{3}] and {1}[{4}]: %{2}\r\n", match.Currency1.Name, match.Currency2.Name, match.CurrentPercentageDifference, match.Currency1.CurrentValue, match.Currency2.CurrentValue);
            match.EntryAlertHasBeenIssued = true;
        }

        private void IssueExitAlert(CurrencyMatch match)
        {
            txtOutput.Text += String.Format("Exit: {0}[{3}] and {1}[{4}]: %{2}\r\n", match.Currency1.Name, match.Currency2.Name, match.CurrentPercentageDifference, match.Currency1.CurrentValue, match.Currency2.CurrentValue);
            match.EntryAlertHasBeenIssued = false;
        }

        private void cmdStart_Click(object sender, EventArgs e)
        {
            timer1.Stop();
            SaveValuesToRegistry();
            InitializeMatches();
            DoTheFrigginWork();
            timer1.Start();
        }

        private void InitializeMatches()
        {
            _matches = new List<CurrencyMatch>() {
                new CurrencyMatch(
                    new InputData(Convert.ToDecimal(txtEurUsdGbpUsd.Text), "EURUSD", "&nbsp;EUR&nbsp;", 70),
                    new InputData(Convert.ToDecimal(txtGbpUsdEurUsd.Text), "GBPUSD", "&nbsp;GBP&nbsp;", 70), 
                    0.5M, lblEurUsdGbpUsdCurrent),
                new CurrencyMatch(
                    new InputData(Convert.ToDecimal(txtEurUsdEurJpy.Text), "EURUSD", "&nbsp;EUR&nbsp;", 70),
                    new InputData(Convert.ToDecimal(txtEurJpyEurUsd.Text), "EURJPY", "&nbsp;EUR&nbsp;", 210), 
                    0.5M, lblEurUsdEurJpyCurrent),
                new CurrencyMatch(
                    new InputData(Convert.ToDecimal(txtGbpUsdGbpJpy.Text), "GBPUSD", "&nbsp;GBP&nbsp;", 70), 
                    new InputData(Convert.ToDecimal(txtGbpJpyGbpUsd.Text), "GBPJPY", "&nbsp;GBP&nbsp;", 210),
                    0.5M, lblGbpUsdGbpJpyCurrent),
                new CurrencyMatch(
                    new InputData(Convert.ToDecimal(txtGbpUsdGbpChf.Text), "GBPUSD", "&nbsp;GBP&nbsp;", 70), 
                    new InputData(Convert.ToDecimal(txtGbpChfGbpUsd.Text), "GBPCHF", "&nbsp;GBP&nbsp;", 350),
                    0.5M, lblGbpUsdGbpChfCurrent),
                new CurrencyMatch(
                    new InputData(Convert.ToDecimal(txtGbpJpyUsdJpy.Text), "GBPJPY", "&nbsp;GBP&nbsp;", 210),
                    new InputData(Convert.ToDecimal(txtUsdJpyGbpJpy.Text), "USDJPY", "&nbsp;USD&nbsp;", 210),
                    0.5M, lblGbpJpyUsdJpyCurrent),
                new CurrencyMatch(
                    new InputData(Convert.ToDecimal(txtUsdJpyEurJpy.Text), "USDJPY", "&nbsp;USD&nbsp;", 210),
                    new InputData(Convert.ToDecimal(txtEurJpyUsdJpy.Text), "EURJPY", "&nbsp;EUR&nbsp;", 210), 
                    0.5M, lblUsdJpyEurJpyCurrent),
                new CurrencyMatch(
                    new InputData(Convert.ToDecimal(txtUsdJpyEurChf.Text), "USDJPY", "&nbsp;USD&nbsp;", 210),
                    new InputData(Convert.ToDecimal(txtEurChfUsdJpy.Text), "EURCHF", "&nbsp;EUR&nbsp;", 350), 
                    0.5M, lblUsdJpyEurChfCurrent)
            };


        }

        private void SaveValuesToRegistry()
        {
            RegistryKey regKey = Registry.CurrentUser.CreateSubKey("EURUSDGBPUSD");
            regKey.SetValue("TheValue", txtEurUsdGbpUsd.Text);
            regKey.Close();
            regKey = Registry.CurrentUser.CreateSubKey("GBPUSDEURUSD");
            regKey.SetValue("TheValue", txtGbpUsdEurUsd.Text);
            regKey.Close();
            regKey = Registry.CurrentUser.CreateSubKey("EURUSDEURJPY");
            regKey.SetValue("TheValue", txtEurUsdEurJpy.Text);
            regKey.Close();
            regKey = Registry.CurrentUser.CreateSubKey("EURJPYEURUSD");
            regKey.SetValue("TheValue", txtEurJpyEurUsd.Text);
            regKey.Close();
            regKey = Registry.CurrentUser.CreateSubKey("GBPUSDGBPJPY");
            regKey.SetValue("TheValue", txtGbpUsdGbpJpy.Text);
            regKey.Close();
            regKey = Registry.CurrentUser.CreateSubKey("GBPJPYGBPUSD");
            regKey.SetValue("TheValue", txtGbpJpyGbpUsd.Text);
            regKey.Close();
            regKey = Registry.CurrentUser.CreateSubKey("GBPUSDGBPCHF");
            regKey.SetValue("TheValue", txtGbpUsdGbpChf.Text);
            regKey.Close();
            regKey = Registry.CurrentUser.CreateSubKey("GBPCHFGBPUSD");
            regKey.SetValue("TheValue", txtGbpChfGbpUsd.Text);
            regKey.Close();
            regKey = Registry.CurrentUser.CreateSubKey("GBPJPYUSDJPY");
            regKey.SetValue("TheValue", txtGbpJpyUsdJpy.Text);
            regKey.Close();
            regKey = Registry.CurrentUser.CreateSubKey("USDJPYGBPJPY");
            regKey.SetValue("TheValue", txtUsdJpyGbpJpy.Text);
            regKey.Close();
            regKey = Registry.CurrentUser.CreateSubKey("USDJPYEURJPY");
            regKey.SetValue("TheValue", txtUsdJpyEurJpy.Text);
            regKey.Close();
            regKey = Registry.CurrentUser.CreateSubKey("EURJPYUSDJPY");
            regKey.SetValue("TheValue", txtEurJpyUsdJpy.Text);
            regKey.Close();
            regKey = Registry.CurrentUser.CreateSubKey("USDJPYEURCHF");
            regKey.SetValue("TheValue", txtUsdJpyEurChf.Text);
            regKey.Close();
            regKey = Registry.CurrentUser.CreateSubKey("EURCHFUSDJPY");
            regKey.SetValue("TheValue", txtEurChfUsdJpy.Text);
            regKey.Close();
        }

        private void GetValuesFromRegistry()
        {
            txtEurJpyEurUsd.Text = ReadRegistryValue("EURJPYEURUSD");
            txtEurUsdEurJpy.Text = ReadRegistryValue("EURUSDEURJPY");
            txtEurUsdGbpUsd.Text = ReadRegistryValue("EURUSDGBPUSD");
            txtGbpJpyGbpUsd.Text = ReadRegistryValue("GBPJPYGBPUSD");
            txtGbpUsdEurUsd.Text = ReadRegistryValue("GBPUSDEURUSD");
            txtGbpUsdGbpJpy.Text = ReadRegistryValue("GBPUSDGBPJPY");
            txtGbpUsdGbpChf.Text = ReadRegistryValue("GBPUSDGBPCHF");
            txtGbpChfGbpUsd.Text = ReadRegistryValue("GBPCHFGBPUSD");
            txtGbpJpyUsdJpy.Text = ReadRegistryValue("GBPJPYUSDJPY");
            txtUsdJpyGbpJpy.Text = ReadRegistryValue("USDJPYGBPJPY");
            txtUsdJpyEurJpy.Text = ReadRegistryValue("USDJPYEURJPY");
            txtEurJpyUsdJpy.Text = ReadRegistryValue("EURJPYUSDJPY");
            txtUsdJpyEurChf.Text = ReadRegistryValue("USDJPYEURCHF");
            txtEurChfUsdJpy.Text = ReadRegistryValue("EURCHFUSDJPY");
        }
        private string ReadRegistryValue(string keyName)
        {
            string retval = String.Empty;
            RegistryKey regKey = Registry.CurrentUser.OpenSubKey(keyName);
            if (regKey != null)
            {
                object value = regKey.GetValue("TheValue");
                if (value != null)
                    retval = value.ToString();
            }
            return retval;
        }

        private void Form1_Load(object sender, EventArgs e)
        {

        }

        private void notifyIcon1_DoubleClick(object sender, EventArgs e)
        {
            this.Show();
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            e.Cancel = true;
            this.Hide();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
    }

    class InputData
    {
        private decimal _startingValue;
        public InputData(decimal startingValue, string name, string startString, int substringLength)
        {
            Name = name;
            StartString = startString;
            SubStringLength = substringLength;
            _startingValue = startingValue;
        }
        public string Name;
        public string StartString;
        public int SubStringLength;
        public decimal CurrentValue;
        public decimal PercentageMoved
        {
            get
            {
                if (_startingValue == 0 || CurrentValue == 0)
                    return 0M;
                decimal movement = CurrentValue - _startingValue;
                return (movement / _startingValue) * 100;
            }
        }
    }

    class CurrencyMatch
    {
        public CurrencyMatch(InputData currency1, InputData currency2, decimal minEntryPercentageDiff, Label currentValueLabel)
        {
            Currency1 = currency1;
            Currency2 = currency2;
            MinEntryPercentageDifference = minEntryPercentageDiff;
            CurrentValueLabel = currentValueLabel;
        }
        public InputData Currency1;
        public InputData Currency2;
        public decimal MinEntryPercentageDifference;
        public decimal CurrentPercentageDifference
        {
            get
            {
                if (Currency1 == null || Currency2 == null || Currency1.PercentageMoved == 0 || Currency2.PercentageMoved == 0)
                    return 0M;
                return Decimal.Round(Math.Abs(Currency1.PercentageMoved - Currency2.PercentageMoved), 4);
            }
        }
        public bool EntryAlertHasBeenIssued = false;
        public bool ExitAlertHasBeenIssued = false;
        public Label CurrentValueLabel;
    }

}
